﻿#region Copyrights Notice
//©2009 Microsoft Corporation.  All rights reserved.
//This portal design is based on the design and some code from DropThings.com which is also licensed under an MS-PL license.
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.InformationSecurity.CISF.Portal.DataAccess;

namespace Microsoft.InformationSecurity.CISF.Portal.Business
{
    public class PortalFacade
    {
        private string _UserName;

        public PortalFacade(string userName)
        {
            this._UserName = userName;
        }
        public UserWidgetSetup RearrangeUserWidgets(UserWidgetSetup _Setup)
        {
            DatabaseHelper dbHelper = new DatabaseHelper();
            //Reorder the new list.
            int row = 0, col = 0;
            int widgetsPerColumn = (int)Math.Ceiling((float)_Setup.RealWeblets.Count / 3.0);
            int widgetsPerRow = 3; 
            
            foreach (RealWeblet w in _Setup.RealWeblets)
            {
                w.TableColPos= col;
                w.TableRowPos = row;

                col++;
                if (col >= widgetsPerRow)
                {
                    col = 0;
                    row++;
                }
                dbHelper.UpdateWidget(_UserName, w.WidgetId, w.TableRowPos, w.TableColPos, w.Expanded, w.Caption, w.State, w.VersionNo);      
            }

            return _Setup;
        }
        private RealWeblet CopyWidgetInstance(Dictionary<string, string> tmpWI)
        {
            RealWeblet rw = new RealWeblet();
            rw.WidgetInstanceId = Int32.Parse(tmpWI["WidgetInstanceId"]);
            rw.WidgetId = Int32.Parse(tmpWI["WidgetId"]);
            rw.UserId = Int32.Parse(tmpWI["UserId"]);
            rw.TableColPos= int.Parse(tmpWI["ColumnNo"]);
            rw.TableRowPos= int.Parse(tmpWI["OrderNo"]);
            rw.SetExpanded(int.Parse(tmpWI["Expanded"]));
            rw.Caption = tmpWI["Title"];
            rw.State = tmpWI["State"];
            rw.VersionNo = int.Parse(tmpWI["VersionNo"]);
            rw.SingleWeblet.WidgetId = Int32.Parse(tmpWI["WidgetId"]);
            rw.SingleWeblet.Url = tmpWI["Url"];

            return rw;
        }

        private SingleWeblet CopyWidget(Dictionary<string, string> tmpW)
        {
            SingleWeblet w = new SingleWeblet();
            w.WidgetId = Int32.Parse(tmpW["WidgetId"]);
            w.Name = tmpW["Name"];
            w.Url = tmpW["Url"];
            w.Description = tmpW["Description"];
            w.CreatedDate = DateTime.Parse(tmpW["CreatedDate"]);
            w.LastUpdate = DateTime.Parse(tmpW["LastUpdate"]);
            w.VersionNo = int.Parse(tmpW["VersionNo"]);
            w.IsDefault = bool.Parse(tmpW["IsDefault"]);
            w.DefaultState = tmpW["DefaultState"];
            w.Icon = tmpW["Icon"];
            w.OrderNo = int.Parse(tmpW["OrderNo"]);

            return w;
        }

        public UserWidgetSetup NewUserVisit()
        {
            using (new AuditTrail(this._UserName, "New user visit"))
            {
                var UserWidgetSetup = new UserWidgetSetup();

                DatabaseHelper dbHelper = new DatabaseHelper();
                //Create a new user
                int UserId = dbHelper.GetUserId(this._UserName);
                //Create default widgets for new user.
                bool retStatus = dbHelper.CreateDefaultWidgets(this._UserName);
                //get list of default widgets just created.
                List<Dictionary<string, string>> RealWeblets = dbHelper.GetUserRealWeblets(this._UserName);

                //Construct UserWidgetSetup...
                foreach (Dictionary<string, string> tmpWI in RealWeblets)
                {
                    RealWeblet rw = CopyWidgetInstance(tmpWI);

                    //Add rw to UserWidgetSetup
                    UserWidgetSetup.RealWeblets.Add(rw);
                }
                return UserWidgetSetup;
            }
        }

        public UserWidgetSetup LoadUserSetup()
        {
            using (new AuditTrail(this._UserName, "Existing user revisit"))
            {
                var UserWidgetSetup = new UserWidgetSetup();

                DatabaseHelper dbHelper = new DatabaseHelper();
                //get list of default widgets for this user.
                List<Dictionary<string, string>> RealWeblets = dbHelper.GetUserRealWeblets(this._UserName);

                //Construct UserWidgetSetup...
                foreach (Dictionary<string, string> tmpWI in RealWeblets)
                {
                    RealWeblet rw = CopyWidgetInstance(tmpWI);

                    //Add rw to UserWidgetSetup
                    UserWidgetSetup.RealWeblets.Add(rw);
                }

                return UserWidgetSetup;
            }
        }
        public UserWidgetSetup GetUserSetup() //Get user setup without auditTrail
        {
            var UserWidgetSetup = new UserWidgetSetup();

            DatabaseHelper dbHelper = new DatabaseHelper();
            //get list of default widgets for this user.
            List<Dictionary<string, string>> RealWeblets = dbHelper.GetUserRealWeblets(this._UserName);

            //Construct UserWidgetSetup...
            foreach (Dictionary<string, string> tmpWI in RealWeblets)
            {
                RealWeblet rw = CopyWidgetInstance(tmpWI);

                //Add rw to UserWidgetSetup
                UserWidgetSetup.RealWeblets.Add(rw);
            }

            return UserWidgetSetup;
        }
        public UserWidgetSetup ResetUserSetup()
        {
            using (new AuditTrail(this._UserName, "Reset user setup"))
            {
                var UserWidgetSetup = new UserWidgetSetup();

                DatabaseHelper dbHelper = new DatabaseHelper();

                //Create default widgets for new user.
                bool retStatus = dbHelper.CreateDefaultWidgets(this._UserName);
                //get list of default widgets just created.
                List<Dictionary<string, string>> RealWeblets = dbHelper.GetUserRealWeblets(this._UserName);

                //Construct UserWidgetSetup...
                foreach (Dictionary<string, string> tmpWI in RealWeblets)
                {
                    RealWeblet rw = CopyWidgetInstance(tmpWI);

                    //Add rw to UserWidgetSetup
                    UserWidgetSetup.RealWeblets.Add(rw);
                }
                return UserWidgetSetup;
            }
        }
        public UserWidgetSetup UpdateUserSetup(UserWidgetSetup _Setup, UserWidgetSetup _newSetup)
        {
            using (new AuditTrail(this._UserName, "Update user setup"))
            {
                DatabaseHelper dbHelper = new DatabaseHelper();

                //find and remove unslected widgets.
                for (int i = 0; i < _Setup.RealWeblets.Count; i++)
                {
                    bool itemFound = false; 
                    foreach(RealWeblet w1 in _newSetup.RealWeblets)
                    {
                        if (w1.WidgetId == _Setup.RealWeblets.ElementAt<RealWeblet>(i).WidgetId)
                        {
                            itemFound = true;
                            break;
                        }
                    }
                    if (itemFound == false) //not found, remove.
                    {
                        bool res = dbHelper.DeleteWidget(this._UserName, _Setup.RealWeblets.ElementAt<RealWeblet>(i).WidgetId);
                        if (res == false) //success
                        {
                            _Setup.RealWeblets.Remove(_Setup.RealWeblets.ElementAt<RealWeblet>(i));
                            i--;
                        }
                    }
                }


                //Find new; add.
                for (int i = 0; i < _newSetup.RealWeblets.Count; i++)
                {
                    bool itemFound = false;
                    foreach (RealWeblet w1 in _Setup.RealWeblets)
                    {
                        if (w1.WidgetId == _newSetup.RealWeblets.ElementAt<RealWeblet>(i).WidgetId)
                        {
                            itemFound = true;
                            break;
                        }
                    }
                    if (itemFound == false) //not found, add.
                    {
                        RealWeblet w = _newSetup.RealWeblets.ElementAt<RealWeblet>(i);

                        w.WidgetId = w.SingleWeblet.WidgetId;
                        w.SetExpanded (1);
                        w.Caption = w.SingleWeblet.Name;
                        w.VersionNo = w.SingleWeblet.VersionNo;
                        w.State = w.SingleWeblet.DefaultState;

                        bool res = dbHelper.AddWidget(_UserName, w.WidgetId, w.TableRowPos, w.TableColPos, w.Expanded, w.Caption, w.State, w.VersionNo);
                        if (res == false) //success
                        {
                            _Setup.RealWeblets.Add(w);
                        }
                    }
                }

                _Setup = RearrangeUserWidgets(_Setup);

                return GetUserSetup();
            }
        }
        public List<SingleWeblet> GetWidgetList()
        {
            using (new AuditTrail(this._UserName, "Get Widget List"))
            {
                List<SingleWeblet> Widgets = new List<SingleWeblet>();     
                DatabaseHelper dbHelper = new DatabaseHelper();
                //get list of widgets.
                List<Dictionary<string, string>> retWidgets = dbHelper.GetWidgetList();

                //Construct UserWidgetSetup...
                foreach (Dictionary<string, string> Wiget in retWidgets)
                {
                    SingleWeblet w = CopyWidget(Wiget);

                    //Add rw to UserWidgetSetup
                    Widgets.Add(w);
                }
                return Widgets;
            }
        }
        public Boolean IsUserRegistered()
        {
            using (new AuditTrail(this._UserName, "Check if user is registered"))
            {
                DatabaseHelper dbHelper = new DatabaseHelper();
                //Create a new user
                Boolean res = dbHelper.IsUserRegistered(this._UserName);
                return res; 
            }
        }
        public Boolean MinimizeWidget(Int32 WidgetId)
        {
            using (new AuditTrail(this._UserName, string.Format("WidgetId : " + WidgetId.ToString() + " ; UserAction: Minimize Widget")))
            {
                DatabaseHelper dbHelper = new DatabaseHelper();
                Boolean res = dbHelper.ModifyWidgetSize(_UserName, WidgetId, 0);
                return res;
            }
        }
        public Boolean MaximizeWidget(Int32 WidgetId)
        {
            using (new AuditTrail(this._UserName, string.Format("WidgetId : " + WidgetId.ToString() + " ; UserAction: Maximize Widget")))
            {
                DatabaseHelper dbHelper = new DatabaseHelper();
                Boolean res = dbHelper.ModifyWidgetSize(_UserName, WidgetId, 1);
                return res;
            }
        }
        public Boolean FullScreenWidget(Int32 WidgetId)
        {
            using (new AuditTrail(this._UserName, string.Format("WidgetId : " + WidgetId.ToString() + " ; UserAction: MaximizeToFullScreen Widget")))
            {
                DatabaseHelper dbHelper = new DatabaseHelper();
                Boolean res = dbHelper.ModifyWidgetSize(_UserName, WidgetId, 2);
                return res;
            }
        }
        public Boolean UpdateWidgetState(Int32 WidgetId, string State)
        {
            using (new AuditTrail(this._UserName, string.Format("WidgetId : " + WidgetId.ToString() + " ; UserAction: Update Widget State")))
            {
                DatabaseHelper dbHelper = new DatabaseHelper();
                Boolean res = dbHelper.SaveWidgetState(_UserName, WidgetId, State);
                return res;
            }
        }
        public Boolean DeleteWidget(Int32 WidgetId)
        {
            using (new AuditTrail(this._UserName, string.Format("WidgetId : " + WidgetId.ToString() + " ; UserAction: Delete Widget")))
            {
                DatabaseHelper dbHelper = new DatabaseHelper();
                Boolean res = dbHelper.DeleteWidget(this._UserName, WidgetId);

                if (res == true) //Error
                    return true;

                var UserWidgetSetup = new UserWidgetSetup();
                //get list of widgets for this user.
                List<Dictionary<string, string>> RealWeblets = dbHelper.GetUserRealWeblets(this._UserName);

                //Construct UserWidgetSetup...
                foreach (Dictionary<string, string> tmpWI in RealWeblets)
                {
                    RealWeblet rw = CopyWidgetInstance(tmpWI);

                    //Add rw to UserWidgetSetup
                    UserWidgetSetup.RealWeblets.Add(rw);
                }


                //rearrange the widgets.
                UserWidgetSetup = RearrangeUserWidgets(UserWidgetSetup);

                if (UserWidgetSetup == null) //Error
                    return true;
                else
                    return false;
            }
        }

    }
}
